@@ -22,7 +22,7 @@
 	
 	;; -------- stddef.h --------
 	(define NULL (c-pointer 0 'void*))
-	(define (c-null? p) (and (c-pointer? p) (equal? p (c-pointer 0 ((object->let p) 'c-type)))))
+	(define (c-null? p) (and (c-pointer? p) (zero? (car (c-pointer->list p)))))
 
 	;; -------- stdbool.h --------
 	(define false #f)
